""" Calnex Paragon Translation module
This file contains functions to translate from Paragon-X style commands
into REST for Paragon-100G and Paragon-neo.

This translation is required specifically in the case where files (such as
reports, captures, data exports) are expected to end up on the user's local
PC. Since the 100G and neo both have browser based UIs, the instrument
itself can't write to the local PC. In this case, the files are downloaded
by the functions here and written to the local PC.

The same mechanism may be used for other purposes at some point in the future.

It is expected that the functions in this file are called from paragon.py
(not directly).

The actual functionality for performing the 100G or neo functions is
contained in calnexRestModified.py. Currently, calnexRestModified.py is a
specially modified version of that delivered in the instrument - they may
be merged in some future release.
"""
##############################################################################
# Copyright (c) Calnex Solutions Ltd 2008 - 2019
# The contents of this file should not be copied or distributed
# without permission being granted by Calnex Solutions Ltd.
#
# All rights reserved.
##############################################################################
# Version History:
#   27 Nov 2019, GM: Added logging

import os
import logging
import calnexRestModified as calnexRest

# ##############################################################################
# Translated paragonset commands:
# Cat Show [neo/100G]: This does nothing for Paragon-neo/100G and so is simply
#   bypassed
# Cat GenerateReport [neo/100G]: Generates the report and downloads it from the
#   instrument into the folder specified
# Cat <fileType> [neo/100G]: File load commands are not required for neo/100G
#   and so these are simply bypassed
#
# Translated paragonget commands:
# Cat <1ppsMetric> OnePpsMiss [X/neo/100G]: The CAT currently does not provide
#   mechanism to return a count of missing 1pps pulses in compatibility
#   mode. This is a new P-X style command to return this count
# Cat AVERAGEDTE cTe [X/neo/P100G]: A bug (CAT-6134) in v24.00 of the CAT
#   causes an "Undefined command" error when fetching cTe results.
#   The wrapper fixes this by calling the equivalent REST command to fetch
#   the result


class CalnexTranslator:
    """ CalnexTranslator class
    Provides methods to interface between the Paragon-X style wrapper
    and the Paragon-Neo / Paragon-100G REST API
    """
    _instrument_type = ""
    _instrument_ip = ""
    _selected_slot = "NA"
    _rest = None
    _logger = None

    def __init__(self, instrument_type, instrument_ip, gui_ip):
        """ Instantiates an instance of the CalnexTranslator class
        Parameters
            instrument_ip: str
                The IP address of the Paragon instrument
        """
        self._instrument_type = instrument_type
        self._instrument_ip = instrument_ip
        if instrument_type == "Paragon-X":
            self._instrument_ip = instrument_ip
            self._rest = calnexRest.px_init("localhost:1337")
        else:
            self._instrument_ip = gui_ip
            self._rest = calnexRest.calnexInit(self._instrument_ip)
        self._logger = logging.getLogger("paragon.translation")

    def cat_set(self, arg_str):
        """
        Parses the incoming command and translates it if necessary.

        Paramters:
            arg_str: str
                The command to be parsed along with any parameters

        Returns:
            Returns True to indicating that the command has been translated
            and no further processing is required. Returns False to indicate
            that the caller should process normally.
        """
        self._logger.debug(arg_str)
        bypass = False

        # print("_cat_set: " + arg_str)

        # Split the arguments into separate tokens
        tokens = arg_str.split(' ')

        if tokens[0] == "Cat":
            # is_cat = True
            cat_command = tokens[1]
            if cat_command == "SelectSlot":
                # If command is SelectSlot, then record the slot for use later
                self._selected_slot = tokens[2]
                bypass = False
            elif (cat_command == "Show" and
                  (self._instrument_type == "Paragon-100G" or
                   self._instrument_type == "Paragon-Neo")):
                # Cat Show is a NOP
                bypass = True
            elif (cat_command == "GenerateReport" and
                  (self._instrument_type == "Paragon-100G" or
                   self._instrument_type == "Paragon-Neo")):
                bypass = True
                fname = os.path.basename(tokens[3])
                folder = os.path.dirname(tokens[3])
                self._rest.calnex_cat_generate_report(tokens[2], fname, folder)
            elif (_is_cat_file_command(cat_command) and
                  (self._instrument_type == "Paragon-100G" or
                   self._instrument_type == "Paragon-Neo")):
                # Cat <fileCommand> is a NOP
                bypass = True

        self._logger.debug("----Bypass: %s", str(bypass))
        return bypass

    def cat_get(self, arg_str):
        """
        Parses the incoming command and translates it if necessary.

        Arguments:
            arg_str	The command to be parsed along with any parameters

        Results:
            Returns two values:
            1. bypass: True to indicating that the command has been translated
            and no further processing is required. Returns False to indicate
            that the caller should process normally.
            2. Value: The response to the get.
            This is only valid when bypass=true
        """
        self._logger.debug(arg_str)

        bypass = False
        done = False
        retval = "NA"
        first_arg = ""

        # print("_cat_get: " + arg_str)

        # Split the arguments into separate tokens
        tokens = arg_str.split(' ')

        if tokens[0] == "Cat":
            cat_command = tokens[1]
            if len(tokens) > 2:
                first_arg = tokens[2]

            # print("Cat command: " + cat_command + " " + first_arg)

            # Cat <metric> OnePpsMiss
            if _is_cat_1pps_metric_command(cat_command):
                # print("1pps metric command " + cat_command + " " + first_arg)
                stat = tokens[2]
                if stat == "OnePpsMiss":
                    bypass = True
                    done = True
                    retval = self._rest.calnex_cat_get_missing_count("1pps")
            # Cat <metric> cTe
            if done is False and first_arg == "cTe":
                bypass = True
                done = True
                retval = self._rest.calnex_cat_get_cte(self._selected_slot)

        self._logger.debug("----Bypass: %s, Response: %s", str(bypass), retval)
        return {"bypass": bypass, "response": retval}

    def cat_wait(self):
        """ Wait for the CAT to complete all current operations """
        self._logger.debug("")
        self._rest.calnex_cat_done()

    def cat_generate_report(self, with_charts, filename):
        """
        The second parameter to Cat GenerateReport is a filename
        For CAT with Paragon-X, the filename should be a fully qualified path
        This ends up with the report being saved on the user's local PC
        For Paragon-100G and Paragon-neo, the instrument has no access to the
        user's PC and the report is generated in <IP>/calnex100g/CAT/Reports
        So, what we will do here is have the file generated on the instrument
        with the specified filename but then copy it to the local PC.
        If no path is specified, then we will copy it into the current
        working directory

        Parameters:
            withCharts: bool
                Indicates whether charts should be included in the report
            filename: str
                The fully qualified filename for the generated report.

        Results:
        The report will be generated and stored in the specified location. If
        the supplied filename has no path or has "./" as the path, then the
        file is stored in the current working directory
        """
        self._logger.debug("%s %s", with_charts, filename)
        folder = os.path.dirname(filename)
        fname = os.path.basename(filename)
        self._rest.calnex_cat_generate_report(with_charts, fname, folder)
        return


def _is_cat_1pps_metric_command(token):
    """ Determines whether any given metric is valid for 1pps measurements
    Parameters
    token: str
        The token following "Cat" in the command

    Returns
    bool
        True if the token is a 1pps metric and False otherwise.
    """
    commands = {"TIMEERROR", "AVERAGEDTE",
                "DTELF", "DTEHF", "DTEMTIE", "DTETDDEV"}
    is_command = False
    # if any(command in token for command in commands):
    if token in commands:
        is_command = True

    return is_command


def _is_cat_file_command(token):
    """
    There are a number of CAT commands related to launching with different file
    types. None of these are relevant for Paragon-100G or Paragon-neo
    This function determines whether the command is one of these file open
    commands.

    Arguments:
        token	The token following "Cat" in the command

    Results:
        Returns True if the token is a file open/close command and
        False otherwise.
    """
    commands = {"Close", "1PPS", "E1T1", "2M", "SyncE",
                "Delta1588ModePacketRate",
                "1588Sync", "1588DelayReq", "1588Rtd", "1588TimeError",
                "1588TCAccuracy", "1588PDV",
                "NtpClientPdv", "NtpServerPdv", "NtpRtd"}
    is_command = False

    if any(command in token for command in commands):
        is_command = True

    return is_command
